package com.gic.haoban.manage.web.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.druid.util.StringUtils;
import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.enterprise.api.dto.EnterpriseAndUserDTO;
import com.gic.enterprise.api.dto.EnterpriseDTO;
import com.gic.enterprise.api.dto.StoreDTO;
import com.gic.enterprise.api.dto.StoreSearchDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.EnterpriseDetailDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseApiService;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.vo.LoginVO;
import com.gic.haoban.manage.web.vo.StoreVo;
import com.gic.wechat.api.dto.qywx.UserDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import com.gic.wechat.token.api.dto.QywxCorpInfoDTO;
import com.gic.wechat.token.api.service.QywxTokenManageService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RestController
public class WxEnterpriseController extends WebBaseController{
	private static Logger logger  = LoggerFactory.getLogger(WxEnterpriseController.class);
	@Autowired
	private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;
	@Autowired
	private EnterpriseService enterpriseService;
	@Autowired
	private WxEnterpriseApiService wxEnterpriseApiService;
	@Autowired
	private DepartmentApiService departmentApiService;
	@Autowired
	private StoreService storeService;
	@Autowired
	private QywxTokenManageService qywxTokenManageService;
	@Autowired
	private Config config;
	@Autowired
	private QywxUserApiService qywxUserApiService;
	
	
	//授权企业列表
	@IgnoreLogin
	@RequestMapping("wx-enterprise-list")
	public HaobanResponse wxEnterpriseList() {
//		String wxEnterpriseId = "123456";
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		List<EnterpriseDetailDTO> list = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
		if(CollectionUtil.isNotEmpty(list)){
			List<String> enterpriseIds = list.stream().map(s->s.getEnterpriseId()).collect(Collectors.toList());
			List<EnterpriseDTO> enterpriseList = enterpriseService.listEnterpriseByIds(enterpriseIds);
			Map<String,EnterpriseDTO> map = com.gic.commons.util.CollectionUtil.toMap(enterpriseList, "enterpriseId");
			for(EnterpriseDetailDTO dto:list){
				EnterpriseDTO enterpriseDTO = map.get(dto.getEnterpriseId());
				if(enterpriseDTO!=null){
					dto.setEnterpriseName(enterpriseDTO.getEnterpriseName());
					dto.setEnterpriseLogo(enterpriseDTO.getQcloudImageUrl());
				}
			}
		}
		return resultResponse(HaoBanErrCode.ERR_1, list);
 
	}
	
	//企业信息
	@IgnoreLogin
	@RequestMapping("wxEnterprise-detail")
	public HaobanResponse enterpriseDetail(String wxEnterpriseId) {
		if(StringUtils.isEmpty(wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_1);
		}
		WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
		
		wxEnterpriseDTO.setSmallVersion("免费版");
		wxEnterpriseDTO.setStoreTotal(departmentApiService.totalStoreCountByEnterpriseId(wxEnterpriseId));
		return resultResponse(HaoBanErrCode.ERR_1,wxEnterpriseDTO);
	}
	
	//企业信息
	@IgnoreLogin
	@RequestMapping("wxEnterprise-refresh")
	public HaobanResponse enterpriseRefresh(String wxEnterpriseId) {
		if(StringUtils.isEmpty(wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_1);
		}
		WxEnterpriseDTO wxEnterpriseDTO = wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(wxEnterpriseDTO == null){
			return resultResponse(HaoBanErrCode.ERR_10012);
		}
		QywxCorpInfoDTO dto = qywxTokenManageService.getCorpInfo(wxEnterpriseDTO.getCorpid(), config.getWxSuiteid());
		Long verifiedEndTime = dto.getVerifiedEndTime();
		if(dto == null){
			return resultResponse(HaoBanErrCode.ERR_10012);
		}
		logger.info("【企业同步】dto={}",JSON.toJSONString(dto));
		WxEnterpriseDTO wxDTO = EntityUtil.changeEntityByJSON(WxEnterpriseDTO.class, dto);
		if(verifiedEndTime != null){
			wxDTO.setVerifiedEndTime(new Date(verifiedEndTime * 1000));
		}
		wxDTO.setWxEnterpriseId(wxEnterpriseId);
		wxEnterpriseApiService.update(wxDTO);
		return resultResponse(HaoBanErrCode.ERR_1,wxEnterpriseDTO);
	}
	//gic企业列表
	@IgnoreLogin
	@RequestMapping("enterprise-search")
	public HaobanResponse gicEnterpriseDetail(String enterpriseName) {
		if(StringUtils.isEmpty(enterpriseName)){
			return resultResponse(HaoBanErrCode.ERR_1);
		}
		EnterpriseAndUserDTO dto = enterpriseService.getEnterpriseAndUserByName(enterpriseName);
		return resultResponse(HaoBanErrCode.ERR_1,dto);
 
	}
	//微信企业绑定接口
	@IgnoreLogin
	@RequestMapping("wx-enterprise-bind")
	public HaobanResponse wxEnterpriseBind(String enterpriseId,String wxEnterpriseId,String version) {
		if(org.apache.commons.lang3.StringUtils.isAnyBlank(enterpriseId,version)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		wxEnterpriseId = login.getWxEnterpriseId();
		int i = wxEnterpriseRelatedApiService.wxEnterpriseBind(enterpriseId,wxEnterpriseId,version);
		if(i==1){
			return resultResponse(HaoBanErrCode.ERR_10006);
		}
		return resultResponse(HaoBanErrCode.ERR_1);
 
	}
	//溢出门店列表
	@IgnoreLogin
	@RequestMapping("store-full-list")
	public HaobanResponse storeFullList(BasePageInfo basePageInfo,String keyWord) {
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
//		String wxEnterpriseId = "1";
		int maxVersionCount = 0;
		List<EnterpriseDetailDTO>  enterpriseList = wxEnterpriseRelatedApiService.listEnterpriseByWxEnterpriseId(wxEnterpriseId);
		if(CollectionUtil.isEmpty(enterpriseList)){
			return resultResponse(HaoBanErrCode.ERR_10010);
		}
		List<String> enterpriseIds = enterpriseList.stream().map(s->s.getEnterpriseId()).collect(Collectors.toList());
		//获取门店列表
		Page page1 = new Page<>();
		page1.setCurrentPage(1);
		page1.setPageSize(Integer.MAX_VALUE);
		StoreSearchDTO storeSearchDTO = new StoreSearchDTO();
		storeSearchDTO.setEnterpriseIdList(enterpriseIds);
		storeSearchDTO.setSearchName(keyWord);
		Page storePage = storeService.storeListPage(page1, storeSearchDTO);
		if(storePage == null || storePage.getResult()==null){
			return resultResponse(HaoBanErrCode.ERR_10009);
		}
		List<StoreDTO> t_list = storePage.getResult();
		List<String> storeIds = t_list.stream().map(s->s.getStoreId()).collect(Collectors.toList());
		if(StringUtils.isEmpty(keyWord)){
			storeIds = null;
		}
		Page<DepartmentDTO> page = departmentApiService.pageFullStoreByWxEnterpriseId(wxEnterpriseId, keyWord,storeIds,maxVersionCount, basePageInfo);
		List<DepartmentDTO> list = page.getResult();
		List<StoreVo> resultList = EntityUtil.changeEntityListByJSON(StoreVo.class,list);
		
		
		
		List<String> t_storeIds = list.stream().map(s->s.getRelatedId()).collect(Collectors.toList());
		String [] storeIdArr =  t_storeIds.toArray(new String[t_storeIds.size()]);
		List<StoreDTO> storeList = storeService.getStores(storeIdArr);
		Map<String,StoreDTO> map = com.gic.commons.util.CollectionUtil.toMap(storeList, "storeId");
		
		for(StoreVo vo:resultList){
			if(map.get(vo.getRelatedId())!=null){
				vo.setStoreCode(map.get(vo.getRelatedId()).getStoreCode());
				vo.setStoreId(map.get(vo.getRelatedId()).getStoreId());
				vo.setStoreName(map.get(vo.getRelatedId()).getStoreName());
			}
		}
		
		
		Page resultPage = new Page<>(); 
		resultPage.setCurrentPage(page.getCurrentPage());
		resultPage.setPageSize(page.getPageSize());
		resultPage.setTotalCount(page.getTotalCount());
		resultPage.setResult(resultList);
		return resultResponse(HaoBanErrCode.ERR_1,resultPage);
		
	}
	
	//微信企业是否设置key
	@IgnoreLogin
	@RequestMapping("is-wx-enterprise-secret-set")
	public HaobanResponse isWxEnterpriseSecretSet() {
		String wxEnterpriseId = "";
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		wxEnterpriseId = login.getWxEnterpriseId();
		WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(wxEnterprise != null && org.apache.commons.lang3.StringUtils.isAnyBlank(wxEnterprise.getContactSecret(),wxEnterprise.getWxSecretKey())){
			return resultResponse(HaoBanErrCode.ERR_1,false);
		}
		return resultResponse(HaoBanErrCode.ERR_1,true);
 
	}
	
	//微信企业绑定接口绑定通讯录key
	@IgnoreLogin
	@RequestMapping("wx-enterprise-contact-secret-set")
	public HaobanResponse wxEnterpriseSecretSet(String contactSecret) {
		if(org.apache.commons.lang3.StringUtils.isAnyBlank(contactSecret)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		String wxEnterpriseId = "";
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		wxEnterpriseId = login.getWxEnterpriseId();
		WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(wxEnterprise == null){
			return resultResponse(HaoBanErrCode.ERR_10012);
		}
		contactSecret = contactSecret.trim();
		UserDTO user = qywxUserApiService.getSelfWorkWxUser(wxEnterprise.getCorpid(), contactSecret, login.getStaffDTO().getWxUserId());
		logger.info("【保存联系人key】user={}",JSON.toJSONString(user));
		if(user != null){
			wxEnterprise.setContactSecret(contactSecret);
			wxEnterprise.setContactSecretLastTime(new Date());
			wxEnterpriseApiService.update(wxEnterprise);
			return resultResponse(HaoBanErrCode.ERR_1,true);
		}else{
			wxEnterprise.setContactSecretLastTime(new Date());
			wxEnterpriseApiService.update(wxEnterprise);
			return resultResponse(HaoBanErrCode.ERR_10013);
		}
 
	}
	
	//微信企业绑定接口绑定通讯录key
	@IgnoreLogin
	@RequestMapping("wx-enterprise-wx-secret-set")
	public HaobanResponse wxEnterpriseWxSecretSet(String wxSecretKey) {
		if(org.apache.commons.lang3.StringUtils.isAnyBlank(wxSecretKey)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		String wxEnterpriseId = "";
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		wxEnterpriseId = login.getWxEnterpriseId();
		WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(wxEnterprise == null){
			return resultResponse(HaoBanErrCode.ERR_10012);
		}
		wxSecretKey = wxSecretKey.trim();
		String res = qywxUserApiService.listSelfExternalUserid(wxEnterprise.getCorpid(), wxSecretKey, login.getStaffDTO().getWxUserId());
		logger.info("【保存联系人key】res={}",res);
		if(res == null){
			wxEnterprise.setWxSecretKeyLastTime(new Date());
			wxEnterpriseApiService.update(wxEnterprise);
			return resultResponse(HaoBanErrCode.ERR_10013);
		}else{
			wxEnterprise.setWxSecretKey(wxSecretKey);
			wxEnterprise.setWxSecretKeyLastTime(new Date());
			wxEnterpriseApiService.update(wxEnterprise);
			return resultResponse(HaoBanErrCode.ERR_1,true);
		}
 
	}
	
	//微信企业会员小程序
	@IgnoreLogin
	@RequestMapping("wx-enterprise-member-secret-set")
	public HaobanResponse wxEnterpriseMemberSecretSet(String memberSecret) {
		if(org.apache.commons.lang3.StringUtils.isAnyBlank(memberSecret)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		String wxEnterpriseId = "";
		LoginVO login = (LoginVO) AuthRequestUtil.getLoginUser();
		wxEnterpriseId = login.getWxEnterpriseId();
		WxEnterpriseDTO wxEnterprise = wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(wxEnterprise == null){
			return resultResponse(HaoBanErrCode.ERR_10012);
		}
		memberSecret = memberSecret.trim();
		UserDTO user = qywxUserApiService.getSelfWorkWxUser(wxEnterprise.getCorpid(), memberSecret, login.getStaffDTO().getWxUserId());
		if(user != null){
			wxEnterprise.setMemberSecret(memberSecret);
			wxEnterprise.setMemberSecretLastTime(new Date());
			wxEnterpriseApiService.update(wxEnterprise);
			return resultResponse(HaoBanErrCode.ERR_1,true);
		}else{
			wxEnterprise.setMemberSecretLastTime(new Date());
			wxEnterpriseApiService.update(wxEnterprise);
			return resultResponse(HaoBanErrCode.ERR_10013);
		}
	}
	
	
	//企业信息
	@RequestMapping("enterprise-is-over")
	public HaobanResponse enterpriseIsOver(String enterpriseId) {
		if(StringUtils.isEmpty(enterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		EnterpriseDTO enterprise = enterpriseService.getEnterpriseByEid(enterpriseId);
		if(enterprise == null || enterprise.getExpireTime() == null){
			return resultResponse(HaoBanErrCode.ERR_1,true);
		}
		if(enterprise.getExpireTime().getTime() > System.currentTimeMillis()){
			return resultResponse(HaoBanErrCode.ERR_1,false);
		}
		return resultResponse(HaoBanErrCode.ERR_1,true);
	}
	
	

}
